@@ -1,6 +1,6 @@
 /* DeScumm - Scumm Script Disassembler (version 2-5 scripts)
  * Copyright (C) 2001  Ludvig Strigeus
- * Copyright (C) 2002-2005  The ScummVM Team
+ * Copyright (C) 2002-2006  The ScummVM Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -16,7 +16,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $Header: /cvsroot/scummvm/tools/descumm.cpp,v 1.109.2.1 2005/10/18 03:04:04 sev Exp $
+ * $URL: https://svn.sourceforge.net/svnroot/scummvm/tools/tags/release-0-9-0/descumm.cpp $
+ * $Id: descumm.cpp 21272 2006-03-14 06:30:03Z kirben $
  *
  */
 
@@ -103,12 +104,12 @@ const char *var_names0[] = {
 	"VAR_HAVE_MSG",
 	/* 4 */
 	"VAR_ROOM",
-	"VAR_5",
+	"VAR_ACTIVE_ACTOR",
 	"VAR_OVERRIDE",
-	"",
+	NULL,
 	/* 8 */
-	"VAR_8",
-	"VAR_9",
+	"VAR_IS_SOUND_RUNNING",
+	"VAR_ACTIVE_VERB",
 	"VAR_CHARCOUNT",
 	NULL
 };
@@ -156,9 +157,9 @@ const char *var_names2[] = {
 	"VAR_VIRT_MOUSE_Y",
 	/* 32 */
 	"VAR_CLICK_AREA",
+	"VAR_CLICK_VERB",
 	NULL,
-	NULL,
-	NULL,
+	"VAR_CLICK_OBJECT",
 	/* 36 */
 	"VAR_ROOM_RESOURCE",
 	"VAR_LAST_SOUND",
@@ -475,25 +476,25 @@ char *get_var(char *buf)
 		
 	assert(i >= 0);
 
-	if (scriptVersion == 0 &&
-			i < ARRAYSIZE(var_names0) && var_names0[i]) {
-		buf += sprintf(buf, var_names0[i]);
-		return buf;
-	} else if (scriptVersion <= 2 &&
-			i < ARRAYSIZE(var_names2) && var_names2[i]) {
-		buf += sprintf(buf, var_names2[i]);
-		return buf;
-	} else if (scriptVersion >= 3 && 
-			i < ARRAYSIZE(var_names3) && var_names3[i]) {
-		buf += sprintf(buf, var_names3[i]);
+	if (scriptVersion >= 5 && 
+			i < ARRAYSIZE(var_names5) && var_names5[i]) {
+		buf += sprintf(buf, var_names5[i]);
 		return buf;
 	} else if (scriptVersion >= 4 && 
 			i < ARRAYSIZE(var_names4) && var_names4[i]) {
 		buf += sprintf(buf, var_names4[i]);
 		return buf;
-	} else if (scriptVersion >= 5 && 
-			i < ARRAYSIZE(var_names5) && var_names5[i]) {
-		buf += sprintf(buf, var_names5[i]);
+	} else if (scriptVersion >= 3 && 
+			i < ARRAYSIZE(var_names3) && var_names3[i]) {
+		buf += sprintf(buf, var_names3[i]);
+		return buf;
+	} else if (scriptVersion >= 1 &&
+			i < ARRAYSIZE(var_names2) && var_names2[i]) {
+		buf += sprintf(buf, var_names2[i]);
+		return buf;
+	} else if (scriptVersion == 0 &&
+			i < ARRAYSIZE(var_names0) && var_names0[i]) {
+		buf += sprintf(buf, var_names0[i]);
 		return buf;
 	} else if ((i & 0x8000) && (GF_UNBLOCKED || ZakFlag))
 		buf += sprintf(buf, "Var[%d Bit %d", (i & 0x0FFF) >> 4, i & 0x000F);
@@ -692,11 +693,11 @@ void do_decodeparsestring_v2(char *buf, byte opcode)
 	*buf = 0;
 }
 
-void do_actorset_v12(char *buf, byte opcode)
+void do_actorops_v12(char *buf, byte opcode)
 {
 	// FIXME: the A2 should be displayed as arg to the subops, not as arg to the
-	// ActorSet itself; but that'll require some more work.
-	buf = do_tok(buf, "ActorSet", ((opcode & 0x80) ? A1V : A1B) | ((opcode & 0x80) ? A2V : A2B) | ANOLASTPAREN);
+	// ActorOps itself; but that'll require some more work.
+	buf = do_tok(buf, "ActorOps", ((opcode & 0x80) ? A1V : A1B) | ((opcode & 0x80) ? A2V : A2B) | ANOLASTPAREN);
 	buf = strecpy(buf, ",[");
 
 	int subop = get_byte();
@@ -706,7 +707,7 @@ void do_actorset_v12(char *buf, byte opcode)
 			buf = do_tok(buf, "Sound", 0);
 			break;
 		case 2:
-			if (ZakFlag && scriptVersion == 1)
+			if (scriptVersion == 1)
 				buf = do_tok(buf, "Color", 0);
 			else
 				buf = do_tok(buf, "Color", A1B);
@@ -800,12 +801,11 @@ void do_actorops(char *buf, byte opcode)
 		case 0x0E:
 			buf = do_tok(buf, "InitAnimNr", ((opcode & 0x80) ? A1V : A1B));
 			break;
-//    case 0x0F: buf=do_tok(buf, "Unk3_Set21_22to1", A1VARUNTIL0xFF); break;
+//    		case 0x0F: buf=do_tok(buf, "PaletteList", A1VARUNTIL0xFF); break;
 		case 0x10:
 			buf = do_tok(buf, "Width", ((opcode & 0x80) ? A1V : A1B));
 			break;
 		case 0x11:
-			// FIXME for GID_MONKEY_VGA / GID_MONKEY_EGA this is:
 			if (scriptVersion == 5)
 				buf = do_tok(buf, "Scale", ((opcode & 0x80) ? A1V : A1B) | ((opcode & 0x40) ? A2V : A2B));
 			else
@@ -1260,7 +1260,6 @@ void do_room_ops(char *buf)
 
 void do_room_ops_old(char *buf, byte master_opcode)
 {
-	/* FIXME - this function is not complete yet! */
 	char	a[256];
 	char	b[256];
 	
@@ -1327,70 +1326,9 @@ void do_room_ops_old(char *buf, byte master_opcode)
 	case 0x06:
 		do_tok(buf, "ShakeOff", 0);
 		break;
-/*
 	case 0x07:
 		do_tok(buf, "Unused", 0);
 		break;
-	case 0x08:
-		buf =
-			do_tok(buf, "roomFunc2",
-						 ((opcode & 0x80) ? A1V : A1B) | ((opcode & 0x40) ? A2V : A2B) |
-						 ((opcode & 0x20) ? A3V : A3B));
-		break;
-	case 0x09:
-		buf = do_tok(buf, "saveLoad?", ((opcode & 0x80) ? A1V : A1B) | ((opcode & 0x40) ? A2V : A2B));
-		break;
-	case 0x0A:
-		buf = do_tok(buf, "screenEffect?", ((opcode & 0x80) ? A1V : A1W));
-		break;
-	case 0x0B:
-		buf =
-			do_tok(buf, "roomFunc2",
-						 ANOLASTPAREN | ((opcode & 0x80) ? A1V : A1W) | ((opcode & 0x40) ?
-																														 A2V : A2W) |
-						 ((opcode & 0x20) ? A3V : A3W));
-		opcode = get_byte();
-		buf =
-			do_tok(buf, NULL,
-						 ASTARTCOMMA | ANOFIRSTPAREN | ((opcode & 0x80) ? A1V : A1B) |
-						 ((opcode & 0x40) ? A2V : A2B));
-		break;
-
-	case 0x0C:
-		buf =
-			do_tok(buf, "roomFunc3",
-						 ANOLASTPAREN | ((opcode & 0x80) ? A1V : A1W) | ((opcode & 0x40) ?
-																														 A2V : A2W) |
-						 ((opcode & 0x20) ? A3V : A3W));
-		opcode = get_byte();
-		buf =
-			do_tok(buf, NULL,
-						 ASTARTCOMMA | ANOFIRSTPAREN | ((opcode & 0x80) ? A1V : A1B) |
-						 ((opcode & 0x40) ? A2V : A2B));
-		break;
-
-	case 0x0D:
-		do_tok(buf, "save-string", ((opcode & 0x80) ? A1V : A1B) | A2ASCII);
-		break;
-	case 0x0E:
-		do_tok(buf, "load-string", ((opcode & 0x80) ? A1V : A1B) | A2ASCII);
-		break;
-
-	case 0x0F:
-		buf = do_tok(buf, "palManipulate", ANOLASTPAREN | ((opcode & 0x80) ? A1V : A1B));
-		opcode = get_byte();
-		buf =
-			do_tok(buf, NULL,
-						 ASTARTCOMMA | ANOFIRSTPAREN | ANOLASTPAREN | ((opcode & 0x80) ?
-																													 A1V : A1B) |
-						 ((opcode & 0x40) ? A2V : A2B));
-		opcode = get_byte();
-		buf = do_tok(buf, NULL, ASTARTCOMMA | ANOFIRSTPAREN | ((opcode & 0x80) ? A1V : A1B));
-		break;
-*/
-	case 0x10:
-		do_tok(buf, "colorCycleDelay", ((opcode & 0x80) ? A1V : A1B) | ((opcode & 0x40) ? A2V : A2B));
-		break;
 	default:
 		// strcpy(buf, "Unknown??");
 		// printf("UGH, unknown room op %d\n", opcode & 0x1F);
@@ -1629,7 +1567,7 @@ void do_unconditional_jump(char *buf)
 	} else {
 		if (num_block_stack && !dontOutputWhile) {
 			BlockStack *p = &block_stack[num_block_stack - 1];
-			if (p->isWhile && cur == p->to)
+			if (p->isWhile && cur == (int)p->to)
 				return;		// A 'while' ends here.
 		}
 		sprintf(buf, "goto %.4X;", to);
@@ -1723,6 +1661,16 @@ void do_if_code(char *buf, byte opcode)
 	emit_if(buf, tmp);
 }
 
+void do_if_active_object(char *buf, byte opcode)
+{
+	char tmp[256];
+
+	int obj = get_byte();
+	sprintf(tmp, "activeObject2 == %d", obj);
+
+	emit_if(buf, tmp);
+}
+
 void do_if_state_code(char *buf, byte opcode)
 {
 	char var[256];
@@ -1731,7 +1679,12 @@ void do_if_state_code(char *buf, byte opcode)
 	int state = 0;
 
 	var[0] = 0;
-	if (scriptVersion > 0) {
+	if (scriptVersion == 0) {
+		if (opcode & 0x40)
+			sprintf(var, "activeObject");
+		else
+			sprintf(var, "%d", get_byte());
+	} else {
 		get_var_or_word(var, opcode & 0x80);
 	}
 
@@ -1955,7 +1908,7 @@ void next_line_V12(char *buf)
 	case 0x93:
 	case 0xD3:
 		// actorSet
-		do_actorset_v12(buf, opcode);
+		do_actorops_v12(buf, opcode);
 		break;
 
 	case 0x2A:
@@ -2625,15 +2578,18 @@ void next_line_V0(char *buf)
 	case 0xD2:
 		do_tok(buf, "actorFollowCamera", ((opcode & 0x80) ? A1V : A1B));
 		break;
-	case 0x15:
-	case 0x55:
 	case 0x95:
 	case 0xD5:
 		do_tok(buf, "actorFromPos",
 					 ((opcode & 0x80) ? A1V : A1B) | ((opcode & 0x40) ? A2V : A2B) | AVARSTORE);
 		break;
+	case 0x15:
+	case 0x55:
+		do_tok(buf, "walkActorToActor",
+					 ((opcode & 0x80) ? A1V : A1B) | ((opcode & 0x40) ? A2V : A2B) | A3B);
+		break;											/* arg1=actor, arg2=actor */
 	case 0x13:
-		do_tok(buf, "lockActor", A1B);
+		do_tok(buf, "lockCostume", A1B);
 		break;
 	case 0x4D:
 		do_tok(buf, "lockRoom", A1B);
@@ -2642,7 +2598,7 @@ void next_line_V0(char *buf)
 		do_tok(buf, "unlockRoom", A1B);
 		break;
 	case 0x93:
-		do_tok(buf, "unlockActor", A1B);
+		do_tok(buf, "unlockCostume", A1B);
 		break;
 	case 0x1A:
 	case 0x9a:
@@ -2799,8 +2755,8 @@ void next_line_V0(char *buf)
 		do_tok(buf, "getRandomNr", AVARSTORE | ((opcode & 0x80) ? A1V : A1B));
 		break;
 
-    case 0x0f:
-    case 0x4f:
+	case 0x0f:
+	case 0x4f:
 		do_tok(buf, "clearState02", ((opcode & 0x40) ? 0 : A1B));
 		break;
 	case 0x37:
@@ -2811,8 +2767,8 @@ void next_line_V0(char *buf)
 	case 0x57:
 		do_tok(buf, "clearState08", ((opcode & 0x40) ? 0 : A1B));
 		break;
-    case 0x8f:
-    case 0xcf:
+	case 0x8f:
+	case 0xcf:
 		do_tok(buf, "setState02", ((opcode & 0x40) ? 0 : A1B));
 		break;
 	case 0xb7:
@@ -2824,23 +2780,23 @@ void next_line_V0(char *buf)
 		do_tok(buf, "setState08", ((opcode & 0x40) ? 0 : A1B));
 		break;
 
-    case 0x3f:
-    case 0xff:
+	case 0x3f:
+	case 0xff:
 		//ifState02;
-    case 0x1f:
-    case 0x5f:
+	case 0x1f:
+	case 0x5f:
 		//ifState04;
-    case 0x2f:
-    case 0x6f:
+	case 0x2f:
+	case 0x6f:
 		//ifState08;
-    case 0x7f:
-    case 0xbf:
+	case 0x7f:
+	case 0xbf:
 		//ifNotState02;
-    case 0x9f:
-    case 0xdf:
+	case 0x9f:
+	case 0xdf:
 		//ifNotState04;
-    case 0xaf:
-    case 0xef:
+	case 0xaf:
+	case 0xef:
 		//ifNotState08;
 		do_if_state_code(buf, opcode);
 		break;
@@ -2893,7 +2849,7 @@ void next_line_V0(char *buf)
 
 	case 0x70:
 	case 0xF0:
-		buf = do_tok(buf, "lights", ((opcode & 0x80) ? A1V : A1B) | A2B | A3B);
+		buf = do_tok(buf, "lights", ((opcode & 0x80) ? A1V : A1B));
 		break;
 	case 0x25:
 	case 0xA5:
@@ -2906,12 +2862,12 @@ void next_line_V0(char *buf)
 		break;
 	case 0x64:
 	case 0xE4:
-		do_tok(buf, "unknown3", A1B);
+		do_if_active_object(buf, opcode);
 		break;
 
 	case 0x30:
 	case 0xB0:
-		do_tok(buf, "loadActor", ((opcode & 0x80) ? A1V : A1B));
+		do_tok(buf, "loadCostume", ((opcode & 0x80) ? A1V : A1B));
 		break;
 
 	case 0x12:
@@ -2969,7 +2925,7 @@ void next_line_V0(char *buf)
 
 	case 0x54:
 	case 0xD4:
-		do_tok(buf, "setObjectName", ((opcode & 0x80) ? A1V : A1B) | A2ASCII);
+		do_tok(buf, "setObjectName", A1B | A2ASCII);
 		break;
 
 	case 0x29:
